/*
  SPDX-FileCopyrightText: 2015-2021 Laurent Montel <montel@kde.org>

  SPDX-License-Identifier: GPL-2.0-only
*/

#ifndef KMFILTERDIALOGTESTGUI_H
#define KMFILTERDIALOGTESTGUI_H

#endif // KMFILTERDIALOGTESTGUI_H
